#include<stdio.h>
#include<stdlib.h>
#include<math.h>

#define Lambda 1		/* US amplitude */
#define C 0.05			/* c */

#define N_CS 1			/* # of CSs, denoted by i */
#define N_proc 2 /* # of processes (ie offset and onset), denoted by j */
#define N_elem 50    /* # of delay elements, denoted by k = 1,...,N */
#define N_step 80		/* time bins/trial, denoted by t */
#define N_trial 50

enum {
  Offset, Onset
};

int _tau[N_CS][N_proc];
double _L[N_step];
int _x[N_step][N_CS][N_proc][N_elem];
int _delta_x[N_step][N_CS][N_proc][N_elem];
double _h[N_step][N_CS][N_proc][N_elem];
double _x_bar[N_step][N_CS][N_proc];
double _delta_E[N_CS][N_proc][N_elem];
double _delta_V[N_CS][N_proc][N_elem];
double _E[N_CS][N_proc][N_elem];
double _V[N_CS][N_proc][N_elem];
double _s[N_step];
double _s_hat[N_step];
double _r[N_step];

int *tau(int i, int j) { return &(_tau[i][j]); }
int *x(int i, int j, int k, int t) { return &(_x[t][i][j][k]); }
int *delta_x(int i, int j, int k, int t) { return &(_delta_x[t][i][j][k]); }
double *h(int i, int j, int k, int t) { return &(_h[t][i][j][k]); }
double *x_bar(int i, int j, int t) { return &(_x_bar[t][i][j]); }
double *L(int t) { return &(_L[t]); }
double *delta_E(int i, int j, int k) { return &(_delta_E[i][j][k]); }
double *delta_V(int i, int j, int k) { return &(_delta_V[i][j][k]); }
double *E(int i, int j, int k) { return &(_E[i][j][k]); }
double *V(int i, int j, int k) { return &(_V[i][j][k]); }
double *s(int t) { return &(_s[t]); }
double *s_hat(int t) { return &(_s_hat[t]); }
double *r(int t) { return &(_r[t]); }

void for_ijkt(void (*func)(int, int, int, int))
{
  int i, j, k, t;
  for(t = 0; t < N_step; t++){	/* t is the most outside variable */
    for(i = 0; i < N_CS; i++){
      for(j = 0; j < N_proc; j++){
	for(k = 0; k < N_elem; k++){
	  func(i, j, k, t);
	}
      }
    }
  }
}
void for_ijk(void (*func)(int, int, int, int), int t)
{
  int i, j, k;
  for(i = 0; i < N_CS; i++){
    for(j = 0; j < N_proc; j++){
      for(k = 0; k < N_elem; k++){
	func(i, j, k, t);
      }
    }
  }
}
void initialize_x(int i, int j, int k, int t) /* Eq.(1) in Desmond and Moore (1988) */
{
  if (*tau(i,j) + k <= t && t < *tau(i,j) + k + 10){
    *x(i,j,k,t) = 1;
  }else{
    *x(i,j,k,t) = 0;
  }
}
double sum_of_Vx(int t)		/* Eq.(2) */
{
  int i, j, k;
  double result = 0;
  for(i = 0; i < N_CS; i++){
    for(j = 0; j < N_proc; j++){
      for(k = 0; k < N_elem; k++){
	result += *V(i,j,k) * *x(i,j,k,t);
      }
    }
  }
  return result;
}
void compute_delta_V(int i, int j, int k, int t) /* Eq.(3) */
{
  *delta_V(i,j,k) = (C * (*L(t) - *s_hat(t))
		     * *h(i,j,k,t) * *x_bar(i,j,t) * *r(t));
}
void initialize_h(int i, int j, int k, int t) /* Eq.(7) */
{
  if (t == *tau(i,j) + k){
    *h(i,j,k,t) = 1;
    return;
  }
  if (t > *tau(i,j) + k){
    *h(i,j,k,t) = 0.8 * *h(i,j,k,t-1);
    return;
  }
  *h(i,j,k,t) = 0;
}
void initialize_x_bar(int i, int j, int k, int t) /* Eq.(8) */
{
  if (*tau(i,j) + 6 < t && t < *tau(i,j) + 25){
    *x_bar(i,j,t) = 0.05 * (t - *tau(i,j)) - 0.25;
    return;
  }
  if (*tau(i,j) + 25 <= t && t < *tau(i,j) + 500){
    *x_bar(i,j,t) = (-1.0/475) * (t - *tau(i,j)) + (500.0/475);
    return;
  }
  *x_bar(i,j,t) = 0;
}
double max_of_Edelta_x(int t)	/* Eq.(4) */
{
  int i, j, k;
  double result = 0;
  for(i = 0; i < N_CS; i++){
    for(j = 0; j < N_proc; j++){
      for(k = 0; k < N_elem; k++){
	if (result < *E(i,j,k) * *delta_x(i,j,k,t)){
	  result = *E(i,j,k) * *delta_x(i,j,k,t);
	}
      }
    }
  }
  return result;
}
void initialize_delta_x(int i, int j, int k, int t) /* Eq.(5) */
{
  if (t != 0 && *x(i,j,k,t) - *x(i,j,k,t-1) == 1){
    *delta_x(i,j,k,t) = 1;
  }else{
    *delta_x(i,j,k,t) = 0;
  }
}
void compute_delta_E(int i, int j, int k, int t) /* Eq.(6) */
{
  *delta_E(i,j,k) = (C * (*L(t) - *r(t))
		     * *delta_x(i,j,k,t) * *x_bar(i,j,t));
}
void initialize_tau(void)
{
  int i, j;
  for(i = 0; i < N_CS; i++){
    for(j = 0; j < N_proc; j++){
      *tau(i,j) = 0;
    }
  }
}
void initialize_L(void)
{
  int t;
  for(t = 0; t < N_step; t++){
    *L(t) = 0;
  }
}
void initialize_E(int i, int j, int k, int t) /* t isn't used */
{
  *E(i,j,k) = 0;
}
void update_E(int i, int j, int k, int t) /* t isn't used */
{
  *E(i,j,k) += *delta_E(i,j,k);
}
void initialize_V(int i, int j, int k, int t) /* t isn't used */
{
  *V(i,j,k) = 0;
}
void update_V(int i, int j, int k, int t) /* t isn't used */
{
  *V(i,j,k) += *delta_V(i,j,k);
}
void output_s_hat(int trial)
{
  int t;
  FILE *file;
  char filename[1024];

  sprintf(filename, "s_hat.dat");
  if (trial == 0){
    file = fopen(filename, "w");
  }else{
    file = fopen(filename, "a");
  }
  for(t = 0; t < N_step; t++){
    fprintf(file, "%d %f\n", t + N_step*trial, *s_hat(t));
  }
  fclose(file);
}
void output_Y(int trial)
{
  double Y[N_step];
  int t, i;
  FILE *file;
  char filename[1024];

  sprintf(filename, "Y.dat");
  if (trial == 0){
    file = fopen(filename, "w");
  }else{
    file = fopen(filename, "a");
  }
  Y[0] = Y[1] = 0;
  for(t = 2; t < N_step; t++){
    Y[t] = 0;
    for(i = 0; i < 2; i++){
      Y[t] += (0.8 * *s_hat(t-i) + 0.2 * Y[t-i-1]) / 3.0;
    }
    if (Y[t] > 1.0){
      Y[t] = 1.0;
    }
    fprintf(file, "%d %f\n", t + N_step*trial, Y[t]);
  }
  fclose(file);
}
void output_V(int trial)
{
  int t, k;
  FILE *file;
  char filename[1024];

  sprintf(filename, "V.dat");
  if (trial == 0){
    file = fopen(filename, "w");
  }else{
    file = fopen(filename, "a");
  }
  for(t = 0; t < N_step; t++){
    for(k = 0; k < N_elem; k++){
      fprintf(file, "%d %d %f\n", t + N_step*trial, k, *V(0,Onset,k));
    }
    fprintf(file, "\n");
  }
  fclose(file);
}
double normalize(double x)
{
  if (x < 0){ return 0; }
  if (x > 1){ return 1; }
  return x;
}
void run_single_trial(void)
{
  int t;

  for(t = 0; t < N_step; t++){
    *s(t) = sum_of_Vx(t) + *L(t); /* output */
    *s_hat(t) = normalize(sum_of_Vx(t));
    *r(t) = max_of_Edelta_x(t);
    for_ijk(compute_delta_E, t);
    for_ijk(compute_delta_V, t);
    for_ijk(update_E, t);
    for_ijk(update_V, t);
  }
}
int main(void)
{
  int trial;

  initialize_tau();
  initialize_L();

  *tau(0,Onset) = 10;
  *tau(0,Offset) = 40;

  for_ijkt(initialize_x);
  for_ijkt(initialize_delta_x);
  for_ijkt(initialize_h);
  for_ijkt(initialize_x_bar);
  for_ijkt(initialize_E);
  for_ijkt(initialize_V);

  for(trial = 0; trial < N_trial; trial++){
    if (trial < N_trial/2){
      *L(40) = Lambda;
    }else{
      *L(40) = 0;
    }      
    run_single_trial();
    output_s_hat(trial);
    output_Y(trial);
    output_V(trial);
  }

  return 0;
}
